package exception_handler;

use Exporter;
use File::Spec;
use File::Basename;
use CGI::Util qw/escape unescape/;
use base qw(Exporter);
use vars qw($VERSION @EXPORT);
no warnings;
use strict;
use Carp;
use Carp qw(cluck);
#use sigtrap;# qw(handler my_error normal-signals stack-trace error-signals);
#use diagnostics;  # maybe toggleable by TM variable ??

our $VERSION = '0.01';
our @EXPORT  = qw(die warn);

BEGIN {
	*CORE::GLOBAL::die = \&__PACKAGE__::die;
	$Carp::CarpLevel = 0;
	my $error_fd = $ENV{"TM_ERROR_FD"};
	open (TM_ERROR_FD, ">&=$error_fd");
}

END {
	close(TM_ERROR_FD);
}
$SIG{__WARN__} = \&syswarn;
$SIG{__DIE__} = \&syserror;

sub id {
	my $level = shift;
	my ($pack, $file, $line) = caller($level);
	my ($dev, $dirs, $id) = File::Spec->splitpath($file);
	return ($file, $line);
}

sub quote {
	my $str = shift;
	$str =~ s/([^A-Za-z0-9])/sprintf("%%%02X", ord($1))/seg;
	return $str;
}

sub my_namecheck {
	my $name = shift;
	return ($name eq "-") ? "untitled" : basename($name);
}

sub syserror {
	my $message = join('', @_);
	chomp($message);
	my ($file, $line) = id(1);
	my $url = "";
	my $display_name = "";
	if ($file ne "-") {
		$display_name = $file;
		$url = 'url=file://' . quote($file) . '&amp;';
	}
	else {
		$url = "";
		$display_name = 'untitled';
	}
	
	if ($message =~ /syntax error/i) {
		$message =~ s/syntax error//gim;
		$message =~ s/at (.*?) line.*?(\d+)/"in <a href=\"txmt:\/\/open?" . $url . "line=$2\">" .  my_namecheck($1) . "<\/a> at line $2"/eg;
		print TM_ERROR_FD "<div id='exception_report' class='framed'>\n";
		print TM_ERROR_FD "<p id='exception'><b>Syntax Error:</b></p>\n";
		print TM_ERROR_FD "<blockquote><table border='0' cellspacing='0' cellpadding='0'>\n";
		print TM_ERROR_FD "<tr><td>$message </td></tr>\n";
		print TM_ERROR_FD "</table></blockquote></div>";
	}
	else {
		$message =~ s/ at (.*)//gim;
		my $where = $1;
		$where =~ s/^(.*?) line.*?(\d+)/"in <a href=\"txmt:\/\/open?" . $url . "line=$2\">" .  my_namecheck($1) . "<\/a> at line $2"/eg;
		print TM_ERROR_FD "<div id='exception_report' class='framed'>\n";
		print TM_ERROR_FD "<p id='exception'><b>System Error:</b> $message</p>\n";
		print TM_ERROR_FD "<blockquote><table border='0' cellspacing='0' cellpadding='0'>\n";
		print TM_ERROR_FD "<tr><td>$where<td></tr>\n" ;
		print TM_ERROR_FD "</table></blockquote></div>";
	}
}

sub syswarn {
	my $message = join('', @_);
	chomp($message);
	my ($file, $line) = id(1);
	my $url = "";
	my $display_name = "";
	if ($file ne "-") {
		$display_name = $file;
		$url = 'url=file://' . quote($file) . '&amp;';
	}
	else {
		$url = "";
		$display_name = 'untitled';
	}
	$message =~ s/ at (.*)//gim;
	my $where = $1;
	$where =~ s/^(.*?) line.*?(\d+)/"in <a href=\"txmt:\/\/open?" . $url . "line=$2\">" .  my_namecheck($1) . "<\/a> at line $2"/eg;
	print TM_ERROR_FD "<div id='exception_report' class='framed'>\n";
	print TM_ERROR_FD "<p id='exception'><b>System Warning:</b> $message</p>\n";
	print TM_ERROR_FD "<blockquote><table border='0' cellspacing='0' cellpadding='0'>\n";
	print TM_ERROR_FD "<tr><td>$where<td></tr>\n" ;
	print TM_ERROR_FD "</table></blockquote></div>";
#	print TM_ERROR_FD "</div>";
}

sub warn {
	my $message = join('', @_);
	chomp($message);
	my ($file, $line) = id(1);
	my $url = "";
	my $display_name = "";
	if ($file ne "-") {
		$display_name = $file;
		$url = 'url=file://' . quote($file) . '&amp;';
	}
	else {
		$url = "";
		$display_name = 'untitled';
	}
	print TM_ERROR_FD "<div id='exception_report' class='framed'>\n";
	print TM_ERROR_FD "<p id='exception'><b>Script Warning:</b> $message</p>\n";
	print TM_ERROR_FD "<blockquote><table border='0' cellspacing='0' cellpadding='0'>\n";
	print TM_ERROR_FD '<tr><td>in <a href="txmt://open?' . $url . 'line=' . $line . '"> ' . $display_name . "</a> at line $line<td></tr>\n" ;
	print TM_ERROR_FD "</table></blockquote></div>";
#	print TM_ERROR_FD "</div>";
}

sub die {
	my $message = join('', @_);
	chomp($message);
	my ($file, $line) = id(1);
	my $url = "";
	my $display_name = "";
	if ($file ne "-") {
		$display_name = $file;
		$url = 'url=file://' . quote($file) . '&amp;';
	}
	else {
		$url = "";
		$display_name = 'untitled';
	}
	print TM_ERROR_FD "<div id='exception_report' class='framed'>\n";
	print TM_ERROR_FD "<p id='exception'><b>Exception:</b> $message</p>\n";
	print TM_ERROR_FD "<blockquote><table border='0' cellspacing='0' cellpadding='0'>\n";
	my $lm = Carp::longmess();
	$lm =~ s/at (.*?) line.*?(\d+)/"in <a href=\"txmt:\/\/open?" . $url . "line=$2\">" .  my_namecheck($1) . "<\/a> at line $2"/eg;
	print TM_ERROR_FD "<tr><td>$lm</td></tr>";
	print TM_ERROR_FD "</table></blockquote></div>";
	print TM_ERROR_FD "</div>";
	print TM_ERROR_FD "<span style='color:red'>Program aborted due to exception!</span><br><br>" if (!$^S);
	exit(1) if (!$^S);
}

return 1;